package com.caimei365.user.service.impl;

import com.caimei365.user.components.CommonService;
import com.caimei365.user.mapper.BaseMapper;
import com.caimei365.user.mapper.LoginMapper;
import com.caimei365.user.model.ResponseJson;
import com.caimei365.user.components.RedisService;
import com.caimei365.user.model.dto.MobileDto;
import com.caimei365.user.model.dto.PasswordDto;
import com.caimei365.user.model.po.ShopPo;
import com.caimei365.user.model.vo.ShopVo;
import com.caimei365.user.model.vo.UserLoginVo;
import com.caimei365.user.model.vo.UserVo;
import com.caimei365.user.service.BaseService;
import com.caimei365.user.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;


/**
 * Description
 *
 * @author : Charles
 * @date : 2021/3/4
 */
@Slf4j
@Service
public class BaseServiceImpl implements BaseService {
    @Resource
    private CommonService commonService;
    @Resource
    private RedisService redisService;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private LoginMapper loginMapper;
    @Value("${spring.cloud.config.profile}")
    private String profile;

    /**
     * 获取图片验证码
     *
     * @param platformType 0:www，1:crm/h5，2:小程序
     */
    @Override
    public ResponseJson<Map<String, Object>> getCaptchaImage(Integer platformType) {
        log.info(profile);
        Map<String, Object> result = new HashMap<>(2);
        try {
            ImageCaptchaUtil.Captcha captcha = null;
            String captchaKey = "";
            while (StringUtils.isBlank(captchaKey)) {
                captcha = ImageCaptchaUtil.getCaptcha(200, 80, 4);
                captchaKey = ImageCaptchaUtil.getCaptchaKey(captcha.getMd5Code(), platformType);
                boolean exists = redisService.exists(captchaKey);
                // 为了防止重复的图片验证码, 存在就循环重新生成
                if (!exists) {
                    // 保存5分钟
                    redisService.set(captchaKey, captcha.getCode(), 5 * 60L);
                    break;
                }
            }
            result.put("baseImage", captcha.getBase64Image());
            result.put("token", captcha.getMd5Code());
            log.info("获取图片验证码:"+captcha.getCode() + ",token:" + captcha.getMd5Code());
            return ResponseJson.success(result);
        } catch (Exception e) {
            log.info(e.getMessage());
            return ResponseJson.error("获取图片验证码异常", null);
        }
    }

    /**
     * 校验图片验证码
     *
     * @param token        图片验证码token
     * @param imgCode      图片验证码
     * @param platformType 0:www，1:crm/h5，2:小程序
     * @return Boolean
     */
    @Override
    public Boolean checkCaptchaImage(String token, String imgCode, Integer platformType) {
        if (StringUtils.isBlank(token) || StringUtils.isBlank(imgCode)){
            return false;
        }
        String captchaKey = ImageCaptchaUtil.getCaptchaKey(token, platformType);
        String captchaCode = (String) redisService.get(captchaKey);
        if (StringUtils.isNotEmpty(captchaCode) && StringUtils.equalsIgnoreCase(imgCode, captchaCode)) {
            // 校验成功
            return true;
        }
        // 不管验证是否成功删除旧的图片验证码
        redisService.remove(captchaKey);
        return false;
    }

    /**
     * 获取短信验证码
     *
     * @param mobile           手机号
     * @param activateCodeType 1:找回密码,2:手机号注册机构,3:供应商注册,4:修改手机号-旧手机验证码,5:修改手机号-新手机验证码
     * @param platformType     0:www，1:crm/h5，2:小程序
     * @param isCheckCaptcha   是否检查图片验证码，0:检查，1:不检查
     * @param imgCode          图片验证码
     * @param token            图片验证码token
     */
    @Override
    public ResponseJson getSmsCode(String mobile, Integer activateCodeType, Integer platformType, Integer isCheckCaptcha, String imgCode, String token) {
        // 验证手机号
        String result = ValidateUtil.validateMobile(mobile);
        if (result != null) {return ResponseJson.error(result);}
        // 检查图片验证码
        if (null == isCheckCaptcha || 0 == isCheckCaptcha){
            Boolean checkFlag = checkCaptchaImage(token, imgCode, platformType);
            if (!checkFlag) {
                return ResponseJson.error("图片验证码错误");
            }
        }
        boolean sendFlag = false;
        // 生成六位验证码
        String randomCode = CodeUtil.generateCodeInt(6);
        boolean isBeta = "dev".equals(profile) || "beta".equals(profile);
        if (isBeta){
            // 开发 和 测试环境 固定短信验证码 666666
            randomCode = "666666";
        }
        String codeTypeTxt = "";
        // 根据手机号查询用户Id
        Integer userId = baseMapper.getUserIdByMobile(mobile);
        if (1 == activateCodeType) {
            if (null == userId || userId < 1) {
                return ResponseJson.error("该手机号尚未注册");
            }
            // 找回密码
            sendFlag = isBeta || AliyunSmsUtil.sendSms(mobile, 1, "{code:" + randomCode + "}");
            codeTypeTxt = "找回密码";
        } else if (2 == activateCodeType || 3 == activateCodeType){
            if (null != userId && userId > 0) {
                return ResponseJson.error("该手机号已被使用");
            }
            if (2 == activateCodeType) {
                // 机构用户(自主)注册
                sendFlag = isBeta || AliyunSmsUtil.sendSms(mobile, 8, "{code:" + randomCode + "}");
                codeTypeTxt = "机构(自主)注册";
            } else if (3 == activateCodeType) {
                // 供应商(自主)注册
                sendFlag = isBeta || AliyunSmsUtil.sendSms(mobile, 9, "{code:"+ randomCode +"}");
                codeTypeTxt = "供应商(自主)注册";
            } else {
                return ResponseJson.error("参数错误：activateCodeType");
            }
        } else if (4 == activateCodeType) {
            // 您正在更换联系人手机号，您的验证码为：${code}。
            sendFlag = isBeta || AliyunSmsUtil.sendSms(mobile, 13, "{code:"+ randomCode +"}");
            codeTypeTxt = "更换联系人(旧手机号验证码)";
        } else if (5 == activateCodeType) {
            if (null != userId && userId > 0) {
                return ResponseJson.error("该手机号已被使用");
            }
            // 您正在更换联系人手机号，您的验证码为：${code}。
            sendFlag = isBeta || AliyunSmsUtil.sendSms(mobile, 13, "{code:"+ randomCode +"}");
            codeTypeTxt = "更换联系人(新手机号验证码)";
        } else {
            return ResponseJson.error("参数错误：activateCodeType");
        }
        if (sendFlag) {
            redisService.set("code:"+mobile, randomCode, 1800L);
            log.info(codeTypeTxt + ",发送到:" + mobile + "的短信验证码为: " + randomCode);
        } else {
            log.info(codeTypeTxt + ",验证码发送失败！");
            return ResponseJson.error("验证码发送失败！请稍后重试");
        }
        return ResponseJson.success("发送验证码成功");
    }

    /**
     * 绑定账号,发送短信验证
     *
     * @param mobile         运营人员手机号
     * @param bindMobile     机构手机号
     * @param platformType   0:www，1:crm/h5，2:小程序
     * @param isCheckCaptcha 是否检查图片验证码，0:检查，1:不检查
     * @param imgCode        图片验证码
     * @param token          图片验证码token
     */
    @Override
    public ResponseJson getBindSmsCode(String mobile, String bindMobile, Integer userId, Integer platformType, Integer isCheckCaptcha, String imgCode, String token) {
        // 验证手机号
        String result = ValidateUtil.validateMobile(mobile);
        if (result != null) {return ResponseJson.error(result);}
        // 检查图片验证码
        if (null == isCheckCaptcha || 0 == isCheckCaptcha){
            Boolean checkFlag = checkCaptchaImage(token, imgCode, platformType);
            if (!checkFlag) {
                return ResponseJson.error("图片验证码错误");
            }
        }
        Integer dbUserId = null;
        if (StringUtils.isNotBlank(bindMobile)){
            result = ValidateUtil.validateMobile(bindMobile);
            if (result != null) {return ResponseJson.error(result);}
            // 根据手机号查询用户Id
            dbUserId = baseMapper.getUserIdByMobile(bindMobile);
        } else if(null != userId) {
            // 根据用户Id查询用户是否存在
            dbUserId = baseMapper.getUserIdByUserId(userId);
        }
        if (null == dbUserId){
            return ResponseJson.error("你输入的账号不存在");
        }
        Integer operationUserId = baseMapper.getOperationUserIdByMobile(mobile);
        if (null != operationUserId){
            // 判断运营人员是否是机构手机号
            Integer tempId = baseMapper.getUserIdByMobile(mobile);
            if (null != tempId && tempId.equals(userId)){
                // 判断机构状态
                Integer clubStatus = baseMapper.getClubStatusByUserId(operationUserId);
                if(null != clubStatus && 91 != clubStatus) {
                    return ResponseJson.error("该手机号已被机构使用");
                }
                // 判断供应商状态
                Integer shopStatus = baseMapper.getShopStatusByUserId(operationUserId);
                if(null != shopStatus && 91 != shopStatus) {
                    return ResponseJson.error("该手机号已被供应商使用");
                }
            }
        }
        // 生成六位验证码
        String randomCode = CodeUtil.generateCodeInt(6);
        boolean isBeta = "dev".equals(profile) || "beta".equals(profile);
        if (isBeta){
            // 开发 和 测试环境 固定短信验证码 666666
            randomCode = "666666";
        }
        String userName = baseMapper.getUserNameByUserId(userId);
        // 欢迎成为${name}的运营人员，您的邀请码为${code}。
        boolean smsFlag = isBeta || AliyunSmsUtil.sendSms(mobile, 11, "{name:"+ userName +",code:"+ randomCode +"}");
        if (!smsFlag) {
            // 短信发送失败重试一次
            smsFlag = AliyunSmsUtil.sendSms(mobile, 11, "{name:\""+ userName +"\",code:"+ randomCode +"}");
        }
        if (smsFlag) {
            redisService.set("code:"+mobile, randomCode, 1800L);
            log.info( "绑定账号,发送到:" + mobile + "的短信验证码为: " + randomCode);
            return ResponseJson.success("发送验证码成功");
        } else {
            return ResponseJson.error("验证码发送失败！请稍后重试");
        }
    }

    /**
     * 获取邮箱验证码
     *
     * @param email  邮箱
     * @param status 1:绑定邮箱,2邮箱注册,3邮箱找回秘密
     */
    @Override
    public ResponseJson getEmailCode(String email, Integer status) {
        // 验证邮箱
        if (!ValidateUtil.validateEmail(email)) {
            return ResponseJson.error("邮箱格式不正确");
        }
        Integer dbUserId = baseMapper.getUserIdByEmail(email);
        try {
            // 生成六位验证码
            String randomCode = CodeUtil.generateCodeInt(6);
            boolean isBeta = "dev".equals(profile) || "beta".equals(profile);
            if (isBeta){
                // 开发 和 测试环境 固定短信验证码 666666
                randomCode = "666666";
            }
            String html = EmailUtil.setMailCode(status, randomCode);
            String subject = "";
            // status 1:绑定邮箱,2邮箱注册,3邮箱找回秘密
            if (status == 1) {
                if (dbUserId != null) {
                    return ResponseJson.error("该邮箱已被使用");
                }
                subject = "采美365网邮箱绑定验证邮件";
            } else if (status == 2) {
                if (dbUserId != null) {
                    return ResponseJson.error("该邮箱已被使用");
                }
                subject = "采美365网注册验证邮件";
            } else if (status == 3) {
                if (dbUserId == null) {
                    return ResponseJson.error("该邮箱尚未注册");
                }
                subject = "采美365网修改密码验证邮件";
            }
            EmailUtil.sendMail(email, subject, html);
            redisService.set("code:"+email, randomCode, 1800L);
            log.info( "subject,发送到:" + email + "的验证码为: " + randomCode);
        } catch (Exception e) {
            return ResponseJson.error("发送验证码失败");
        }
        return ResponseJson.success("发送验证码成功", "");
    }

    /**
     * 修改密码（找回密码）
     *
     * @param passwordDto {
     *                    mobileOrEmail 手机号或邮箱
     *                    smsCode       短信验证码(旧：activationCode)
     *                    password      密码
     *                    confirmPwd    确认密码
     *                    status        1:手机号找回,2:邮箱找回
     *                    }
     */
    @Override
    public ResponseJson updatePassword(PasswordDto passwordDto) {
        String mobileOrEmail = passwordDto.getMobileOrEmail();
        String smsCode = passwordDto.getSmsCode();
        String passWord = passwordDto.getPassword();
        String confirmPwd = passwordDto.getPasswordConfirm();
        Integer status = passwordDto.getStatus();
        if (status == 1) {
            // 验证手机号
            String result = ValidateUtil.validateMobile(mobileOrEmail);
            if (result != null) {return ResponseJson.error(result);}
        } else {
            // 验证邮箱
            if (!ValidateUtil.validateEmail(mobileOrEmail)) {
                return ResponseJson.error("邮箱格式不正确");
            }
        }
        if (StringUtils.isEmpty(smsCode)) {
            return ResponseJson.error("请输入验证码");
        }
        String redisSmsCode = (String) redisService.get("code:" + mobileOrEmail);
        // 开发 和 测试环境 固定短信验证码 666666
        if ("dev".equals(profile) || "beta".equals(profile)){
            redisSmsCode = (null != redisSmsCode && !"null".equals(redisSmsCode) ? redisSmsCode : "666666");
        }
        if (!smsCode.equals(redisSmsCode)) {
            return ResponseJson.error("验证码错误");
        }
        // 查找用户表是否存在
        Integer dbUserId;
        if (status == 1) {
            dbUserId = baseMapper.getUserIdByMobile(mobileOrEmail);
            if (dbUserId == null) {
                return ResponseJson.error("该手机号尚未注册");
            }
        } else {
            dbUserId = baseMapper.getUserIdByEmail(mobileOrEmail);
            if (dbUserId == null) {
                return ResponseJson.error("该邮箱尚未注册");
            }
        }
        if (StringUtils.isEmpty(passWord)) {
            return ResponseJson.error("请输入新密码");
        }
        if (StringUtils.isEmpty(confirmPwd)) {
            return ResponseJson.error("请确认密码");
        }
        if (!passWord.equals(confirmPwd)) {
            return ResponseJson.error("两次输入的密码不一致");
        }
        // 设置密码
        String dbPassword = Md5Util.md5(passWord);
        baseMapper.updatePasswordByUserId(dbPassword, dbUserId);
        return ResponseJson.success("密码修改成功", "");
    }

    /**
     * 更换手机号
     *
     * @param mobileDto {
     *                  mobile          旧手机号(contractMobile)
     *                  smsCode         旧手机短信验证码(mobileCode)
     *                  newMobile       新手机号(contractMobile2)
     *                  newSmsCode      新手机短信验证码(mobileCode)
     *                  userId          用户Id
     *                  }
     */
    @Override
    public ResponseJson updateMobile(MobileDto mobileDto) {
        // 参数校验
        if (StringUtils.isBlank(mobileDto.getMobile())) {
            return ResponseJson.error("参数异常：旧手机号不能为空！");
        }
        if (StringUtils.isBlank(mobileDto.getNewMobile())) {
            return ResponseJson.error("参数异常：新手机号不能为空！");
        }
        if (StringUtils.isBlank(mobileDto.getSmsCode())) {
            return ResponseJson.error("参数异常：旧手机号验证码不能为空！");
        }
        if (StringUtils.isBlank(mobileDto.getNewSmsCode())) {
            return ResponseJson.error("参数异常：新手机号验证码不能为空！");
        }
        if (null == mobileDto.getUserId()) {
            return ResponseJson.error("参数异常：用户Id不能为空！");
        }
        // 旧手机号与验证码校验
        String result = ValidateUtil.validateMobile(mobileDto.getMobile());
        if (result != null) {return ResponseJson.error(result);}
        String smsCode = (String) redisService.get("code:" + mobileDto.getMobile());
        // 开发 和 测试环境 固定短信验证码 666666
        if ("dev".equals(profile) || "beta".equals(profile)){
            smsCode = (null != smsCode && !"null".equals(smsCode) ? smsCode : "666666");
        }
        if (!mobileDto.getSmsCode().equals(smsCode)) {
            return ResponseJson.error("旧手机验证码错误");
        }
        UserLoginVo user = loginMapper.getLoginUserByUserId(mobileDto.getUserId());
        if (null == user) {
            return ResponseJson.error("账户异常");
        }
        //验证新手机号是否可用
        result = commonService.mobileAndCodeValidate(mobileDto.getNewMobile(), mobileDto.getNewSmsCode());
        if (result != null) {return ResponseJson.error(result);}
        // 更新user表手机号
        baseMapper.updateMobileByUserId(mobileDto.getNewMobile(), mobileDto.getUserId());
        if (user.getUserIdentity() == 3) {
            //供应商修改手机号
            baseMapper.updateShopMobileByShopId(mobileDto.getNewMobile(), user.getShopId());
        } else {
            //供应商修改手机号
            baseMapper.updateClubMobileByClubId(mobileDto.getNewMobile(), user.getClubId());
        }
        return ResponseJson.success();
    }

}
